<?php

namespace Drupal\donorperfect\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\donorperfect\DPUtility;

/**
 * Extends ContentEntityBase for entities stored in DonorPerfect.
 *
 * @ingroup donorperfect
 */

class EntityBase extends ContentEntityBase {

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = [];
    $field_infos = DPUtility::getEntityFieldInfo($entity_type->id());
    $id_key = $entity_type->getKey('id');
    foreach ($field_infos as $field_info) {
      if (!empty($field_info) && is_array($field_info) && !empty($field_info['dp_field']) && !empty($field_info['entity_data_type'])) {
        $field = BaseFieldDefinition::create($field_info['entity_data_type'])
          ->setLabel($field_info['prompt'] ? new TranslatableMarkup($field_info['prompt']) : $field_info['dp_field'])
          ->setReadOnly($field_info['calculated'] ?? FALSE)
          ->setCardinality($field_info['multi'] ? FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED : 1)
          ->setSetting('dp_table', $field_info['dp_table'])
          ->setSetting('dp_field', $field_info['dp_field'])
          ->setSetting('dp_quote_value', ($field_info['quote'] ?? TRUE));
        if ($field_info['dp_field'] === $id_key) {
          $field->setReadOnly(TRUE)
            ->setSetting('unsigned', TRUE);
        }
        switch ($field_info['entity_data_type']) {
          case 'string':
            if (!empty($field_info['max_length']) && ((int)$field_info['max_length'] > 0)) {
              $field->setSetting('max_length', (int)$field_info['max_length']);
            }
            break;
          case 'list_string':
            $field->setSetting('allowed_values', DPUtility::getCodesAsOptions($field_info['dp_field'], TRUE));
            break;
          case 'datetime':
            $field->setSetting('datetime_type', 'date');
            break;
        }
        $fields[$field_info['dp_field']] = $field;
      }
    }
    return $fields;
  }

  /**
   * Return the value of specified entity field.
   *
   * @param string $field_name
   * @param array $options
   *
   * @return mixed
   */
  public function getFieldValue(string $field_name, array $options=[]) {
    try {
      $value = $this->get($field_name);
    } catch (\InvalidArgumentException $e) {
      return null;
    }
    if (!empty($value)) {
      if ($date = $value->date) {
        return $date;
      } elseif (isset($value->value)) {
        return $value->value;
      } elseif (!empty($value->target_id)) {
        $entities = $value->referencedEntities();
        if (empty($entities)) {
          return null;
        } elseif (count($entities) === 1) {
          return reset($entities);
        } else {
          if (!empty($options['return_first'])) {
            return reset($entities);
          } else {
            return $entities;
          }
        }
      }
    } else {
      return null;
    }
  }

}
